@@ -19,7 +19,7 @@
 # Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
 # additional non-glade widgets and misc stuff
-_WDEBUG = 1
+_WDEBUG = 0
 import os,sys
 import pygtk
 #this is needed for py2exe
@@ -38,6 +38,8 @@ from SimpleGladeApp import SimpleGladeApp
 
 app_name = "gvr_gtk"
 import utils
+
+from worldMap import lookup_dir_dict
 ##locale_dir = utils.LOCALEDIR 
 ##bindtextdomain(app_name, locale_dir)
 #utils.set_locale()
@@ -134,11 +136,14 @@ class Canvas(gtk.DrawingArea):
         # image sizes
         self.spi_x = self.splash_pixbuf.get_width()
         self.spi_y = self.splash_pixbuf.get_height()
+        
         # all guidos are the same size and square
         self.guido_x = self.robot_n_pixbuf.get_width()
         self.guido_y = self.guido_x
         # size of the matrix cells
         self.square = 40
+        self.offset = (0,0)
+        
         # 'stuff_to_draw' will hold references to the drawing methods.
         # They are set by the method draw_world and draw_splash.
         # The 'on_expose_event' callback just
@@ -150,7 +155,7 @@ class Canvas(gtk.DrawingArea):
         # The gui should call draw_world. The list
         # is parsed from item 0 upwards.
         self.stuff_to_draw = [self._draw_splash]
-        
+                
     def __repr__(self):
         return "Canvas"
         
@@ -184,7 +189,11 @@ class Canvas(gtk.DrawingArea):
         # tell the layout which font description to use
         self.pangolayout.set_font_description(font_desc)
         # and one for the beepers
-        font_desc = pango.FontDescription('Serif 12')
+        if utils.platform == 'XO':
+            fn_size = '8'
+        else:
+            fn_size = '12'
+        font_desc = pango.FontDescription('Serif '+fn_size)
         self.pangolayout_beeper.set_font_description(font_desc)
         return True
         
@@ -192,6 +201,8 @@ class Canvas(gtk.DrawingArea):
         self.width = allocation.width
         self.height = allocation.height
         #print 'x,y', self.width,self.height
+        self.screenX = self.width/self.square
+        self.screenY = self.height/self.square
         return True
         
     def _on_expose_event(self, widget, event):
@@ -210,17 +221,22 @@ class Canvas(gtk.DrawingArea):
     
     def _draw_splash(self,widget):
         """Draws the GvR splash screen onto the canvas."""
-        # set_size_request is to force the scrolledwindow to link to the
-        # widget size, meaning it will work like expected :-)
-        # So call the request with the desired size of the 'canvas'
-        widget.set_size_request(self.spi_x,self.spi_y)
+        self.world_size = (10,10)
+        x = self.world_size[0] * self.square
+        y = self.world_size[1] * self.square
+        self.set_size_request(x,y)
         self._fill_background(widget)
-        offset_x = (self.width - self.spi_x)/2
-        offset_y = (self.height - self.spi_y)/2
+##        offset_x = (self.width - self.spi_x)/2
+##        offset_y = (self.height - self.spi_y)/2
         widget.window.draw_pixbuf(
-                    self.gc,self.splash_pixbuf,0,0,offset_x,offset_y,-1,-1)
+                    self.gc,self.splash_pixbuf,0,0,8,8,-1,-1)
     
     # these methods are used to blit the world for the first time   
+    def _reset_offset(self):
+        self.offset = (0,0)
+        self.screenX = self.width/self.square
+        self.screenY = self.height/self.square
+    
     def _draw_empty_world(self,widget):
         if repr(self) == "WBCanvas":
             col = self.BLUE
@@ -228,8 +244,7 @@ class Canvas(gtk.DrawingArea):
             col = self.RED
         #self.logger.debug("_draw_empty_world called")
         self._fill_background(widget)
-        # get start coords, meaning the pos of the robot
-        offset_x,offset_y = self.world.robot# not used yet XXX
+        
         self.gc.set_line_attributes(line_width=2,
                                     line_style=gtk.gdk.LINE_SOLID,
                                     cap_style=gtk.gdk.CAP_NOT_LAST,
@@ -239,66 +254,90 @@ class Canvas(gtk.DrawingArea):
         # tell the layout which font description to use
         self.pangolayout.set_font_description(font_desc)
         # set numbers and outer walls
-        square = self.square# locals are quicker then globals
-        y = self.height - square + 4
-        step = square
+        y = self.height - self.square + 4
+        step = self.square
         # self.orig_x, self.orig_y are the coords of the upper left corner of
         # guido at the first square in the matrix. 
         # Used as starting point to calculate the robots and beepers position.
-        self.orig_x, self.orig_y = step+9,y-square+6
+        self.orig_x, self.orig_y = step+9,y-self.square+6
         #print self.orig_x,self.orig_y
         #widget.window.draw_pixbuf(self.gc,self.dot_pixbuf,0,0,self.orig_x,self.orig_y,-1,-1)
-        end = self.width/square
+        end = self.width/self.square
         # draw horizontal outer red wall
         self.gc.set_foreground(col)
         widget.window.draw_line(self.gc,step+4,y,self.width,y)
-        # draw horizontal labels
-        self.gc.set_foreground(self.BLACK)
-        step += square/2
-        y += 4
-        # Here we set some pygtk methods to the local namespace for 
-        # better speed
-        window_draw_layout = widget.window.draw_layout
-        window_draw_pixbuf = widget.window.draw_pixbuf
-        pangolayout_set_text = self.pangolayout.set_text
-        for x in range(1,end):
-            pangolayout_set_text('%d' % x)
-            window_draw_layout(self.gc,
-                                step, y,
-                                self.pangolayout)
-            step += square
-        step = self.height- square*2
+        step = self.height- self.square*2
         # draw vertical outer red wall
         self.gc.set_foreground(col)
-        widget.window.draw_line(self.gc,square+4,step+square+4,square+4,0)
+        widget.window.draw_line(self.gc,self.square+4,step+self.square+4,self.square+4,0)
         #draw vertical labels
         self.gc.set_foreground(self.BLACK)
-        x_range_dots = range(int(square*2),self.width,square)
+        x_range_dots = range(int(self.square*2),self.width,self.square)
         for y in range(1,self.height):
-            pangolayout_set_text('%d' % y)
-            window_draw_layout(self.gc,
-                                square/2, step+square/2,
-                                self.pangolayout)
             # draw dots on x-axes
             for x in x_range_dots:
-                window_draw_pixbuf(self.gc,self.dot_pixbuf,0,0,x,step,-1,-1)
-            step -= square
+                widget.window.draw_pixbuf(self.gc,self.dot_pixbuf,0,0,x,step,-1,-1)
+            step -= self.square
         return True
-        
+    
+    def _draw_labels(self,widget=None):
+        # used to determine the amount the world must shift as the robot moves
+        # off screen
+        offset_x,offset_y = self.offset
+        # create a font description
+        font_desc = pango.FontDescription('Serif 8')
+        # tell the layout which font description to use
+        self.pangolayout.set_font_description(font_desc)
+        # draw horizontal labels
+        self.gc.set_foreground(self.BLACK)
+        end = self.width/self.square
+        y = self.height - self.square + 8
+        step = self.square 
+        for x in range(1+offset_x,end+offset_x+1):
+            self.pangolayout.set_text('%d' % x)
+            self.window.draw_layout(self.gc,
+                                step, y,
+                                self.pangolayout)
+            step += self.square
+        # vertical labels
+        step = self.height- self.square*2
+        for y in range(1+offset_y,self.height+offset_y):
+            self.pangolayout.set_text('%d' % y)
+            self.window.draw_layout(self.gc,
+                                self.square/2, step+self.square/2,
+                                self.pangolayout)
+            step -= self.square
+            
     def _draw_walls(self,widget):
         #self.logger.debug("_draw_walls called")
         self.gc.set_foreground(self.RED)
         self.gc.line_width = 3
         walls = self.world.get_walls_position()
+        # used to determine the amount the world must shift as the robot moves
+        # off screen
+        offset_x,offset_y = self.offset
         for x,y in walls['west_wall']:
             x = x * self.square + 4
-            y = self.height - self.square - y*self.square + 4
-            widget.window.draw_line(self.gc,x,y,x,y+self.square)
+            y = self.height - self.square - y*self.square + 8
+            widget.window.draw_line(self.gc,x,y,x,y+(self.square-8))
         for x,y in walls['south_wall']:
-            x = x * self.square + 4
+            x = x * self.square + 8
             y = self.height - y*self.square + 4
-            widget.window.draw_line(self.gc,x,y,x+self.square,y)
-            
+            widget.window.draw_line(self.gc,x,y,x+(self.square-8),y)
+    
+    # Used by the worldbuilder
+    def _remove_wall(self,d,x,y):
+        self.gc.set_foreground(self.WHITE)
+        self.gc.line_width = 3
+        if d == 'W':
+            x = x * self.square + 4
+            y = self.height - self.square - y*self.square + 8
+            self.window.draw_line(self.gc,x,y,x,y+(self.square-8))
+        elif d == 'S':
+            x = x * self.square + 8
+            y = self.height - y*self.square + 4
+            self.window.draw_line(self.gc,x,y,x+(self.square-8),y)
+
     def _draw_beepers(self,widget):
         #self.logger.debug("_draw_beepers called")
         self.gc.set_foreground(self.BLUE)
@@ -312,6 +351,10 @@ class Canvas(gtk.DrawingArea):
         #self.logger.debug("_draw_beeper called")
         pos_x = self.orig_x + self.square*(pos[0]-1)
         pos_y = self.orig_y - self.square*(pos[1]-1)
+        # used to determine the amount the world must shift as the robot moves
+        # off screen
+        offset_x,offset_y = self.offset
+        
         self.window.draw_arc(self.gc,False,pos_x+2,pos_y+2,22,24,0,360*64)
         
         self.pangolayout_beeper.set_text('%d' % value)
@@ -327,9 +370,11 @@ class Canvas(gtk.DrawingArea):
         #self.logger.debug("_draw_robot called")
         pos = self.world.get_robots_position()
         dir = self.world.get_robots_direction()
+##        # used to determine the amount the world must shift as the robot moves
+##        # off screen
+##        offset_x,offset_y = self.offset
         pos_x = self.orig_x + self.square*(pos[0]-1)
-        pos_y = self.orig_y - self.square*(pos[1]-1)
-        #print '_draw_robot',pos
+        pos_y = self.orig_y - self.square*(pos[1]-1)            
         pixbuf = self._get_direction_pixbuf()
         widget.window.draw_pixbuf(self.gc,pixbuf,0,0,pos_x,pos_y,-1,-1)
             
@@ -345,10 +390,20 @@ class Canvas(gtk.DrawingArea):
         """Draws the complete world represented in @world"""
         self.world = world
         self.stuff_to_draw = [self._draw_empty_world,
+                                self._draw_labels,
                                 self._draw_walls,
                                 self._draw_robot,
                                 self._draw_beepers]
         self.queue_draw()
+########################## work in progress    
+    def draw_scrolling_world(self,offset):
+        self.logger.debug("draw_scrolling_world called")
+        self.stuff_to_draw = [self._draw_empty_world,\
+                                self._draw_walls,\
+                                self._draw_beepers,\
+                                self._draw_labels]
+        self.queue_draw()
+#########################################################        
     def draw_robot(self,obj,oldcoords):
         """Draws the robot and clears the old position of the robot.
         This is more efficient then redraw the whole world every time."""
@@ -360,19 +415,35 @@ class Canvas(gtk.DrawingArea):
         dir = self.world.get_robots_direction()
         # we don't use a function for calculating the positions because
         # of the overhead of function calling. Speed is important in this case.
-        pos_x = self.orig_x + self.square*(pos[0]-1)
-        pos_y = self.orig_y - self.square*(pos[1]-1)
+        pos_x = self.orig_x + self.square*(pos[0] + self.offset[0] - 1)
+        pos_y = self.orig_y - self.square*(pos[1] - self.offset[1] - 1)
+############### Scrolling is work in progress
+        #print 
         #print 'draw_robot',pos,oldcoords
+        scrolling, self.offset = self.world.newOffset(self.offset, (self.screenX,self.screenY))
+        
+        #print 'scrolling',scrolling,'self.offset',self.offset
+        if scrolling:
+            self.screenX += self.offset[0]
+            self.screenY += self.offset[1]
+            self.draw_scrolling_world(self.offset)
+            # recalculate the robot's position
+##            pos_x = self.orig_x + self.square*(pos[0] + self.offset[0] - 1)
+##            pos_y = self.orig_y - self.square*(pos[1] - self.offset[1] - 1)
+            
+###########################################
         pixbuf = self._get_direction_pixbuf()
         self.gc.set_foreground(self.WHITE)
         if oldcoords:
-            y = self.orig_y - self.square*(oldcoords[1]-1)
-            x = self.orig_x + self.square*(oldcoords[0]-1)
+            y = self.orig_y - self.square*(oldcoords[1]- self.offset[1] -1)
+            x = self.orig_x + self.square*(oldcoords[0]+ self.offset[0] -1)
             self.window.draw_rectangle(self.gc,True,x,y,
                                     self.guido_x,self.guido_y)
             beepersdict = self.world.get_beepers()
             if beepersdict.has_key(pos):
                 self.draw_beepers(None)
+        # make sure all the previous events are processed before drawing the robot
+        #while gtk.events_pending(): gtk.main_iteration()
         self.window.draw_pixbuf(self.gc,pixbuf,0,0,pos_x,pos_y,-1,-1)
         ##self.queue_draw()
         
@@ -536,6 +607,11 @@ class WBCanvas(Canvas):
         # code used by button 1 and 3    
         wcode = filter(None,[wline,bline])
         if wcode:
+##            self.stuff_to_draw = [self._draw_empty_world,
+##                                self._draw_robot,
+##                                self._draw_beepers,
+##                                self._draw_labels,
+##                                self._draw_walls]
             #print wcode
             if wcode[0] in self.wcode:
                 #print "double:", wcode[0]
@@ -543,7 +619,18 @@ class WBCanvas(Canvas):
             else:                
                 self.wcode = self.wcode + wcode
             self.gvrparent.world_editor.editor.set_text(self.wcode)
-            self.gvrparent.on_button_reload()
+            if event.button == 1:
+                # t isn't used
+                t,x,y,d = wline.split(' ')
+                result = self.world.setWall_wb(x,y,lookup_dir_dict[d[:-1]])
+                #print 'result',result
+                if result[2] == 0:
+                    # remove wall
+                    self._remove_wall(result[0],result[1][0],result[1][1])
+                else:
+                    self._draw_walls(self)
+            else:
+                self.gvrparent.on_button_reload()
         return True
         
 # setup the timer object the model can use
