@@ -75,6 +75,8 @@ class Window(SimpleGladeApp):
         if PLATFORM != 'XO':
             #self.parentGUI = gtk.Window(gtk.WINDOW_TOPLEVEL)
             self.parentGUI = self.window_main
+            self.windowtitle = "Gvrng"
+            self.parentGUI.set_title(self.windowtitle)
             file = os.path.join(os.getcwd(),'gui-gtk','pixmaps','gvrIcon.bmp')
             try:
                 self.parentGUI.set_icon_from_file(file)
@@ -183,9 +185,9 @@ class Window(SimpleGladeApp):
         self.program_editor = None
         
     ## These are the callbacks mandatory for the controller
-    def start(self,args):
+    def start(self,*args):
         """This will start the GUI."""
-        self.logger.debug("start called with args: %s,%s" % args)
+        self.logger.debug("start called with args: %s" % args)
         # there are only args when gvr is started by gvrng.py (non-XO systems)
         if args[0] and args[1]:
             wfile, pfile = args[0], args[1]
