@@ -23,11 +23,18 @@ import logging
 import sys,os,__builtin__,ConfigParser
 import utils
 
+# commandline arguments are used for developing only
+if len(sys.argv) >= 2 and sys.argv[1] == '--debug':
+    CONSOLELOGLEVEL = logging.DEBUG
+    FILELOGLEVEL = logging.DEBUG
+else:
+    CONSOLELOGLEVEL = logging.ERROR
+    FILELOGLEVEL = None
+
 PLATFORM = utils.platform
 
 if PLATFORM != 'XO':
-    CONSOLELOGLEVEL = logging.DEBUG
-    FILELOGLEVEL = logging.DEBUG
+    fh = None
     LOGPATH = 'gvr.log'
     #create logger
     logger = logging.getLogger("gvr")
@@ -35,18 +42,21 @@ if PLATFORM != 'XO':
     #create console handler and set level to error
     ch = logging.StreamHandler()
     ch.setLevel(CONSOLELOGLEVEL)
-    #create file handler and set level to debug
-    fh = logging.FileHandler(LOGPATH)
-    fh.setLevel(FILELOGLEVEL)
+    if FILELOGLEVEL:
+        #create file handler and set level to debug
+        fh = logging.FileHandler(LOGPATH)
+        fh.setLevel(FILELOGLEVEL)
     #create formatter
     formatter = logging.Formatter("%(asctime)s - %(name)s - %(levelname)s - %(message)s")
     #add formatter to ch and fh
     ch.setFormatter(formatter)
-    fh.setFormatter(formatter)
     #add ch and fh to logger
+    if fh:
+        fh.setFormatter(formatter)
+        logger.addHandler(fh)
     logger.addHandler(ch)
-    logger.addHandler(fh)
-
+    if fh:
+        logger.debug("Starting logfile: %s" % os.path.join(os.getcwd(),LOGPATH))
         
 module_logger = logging.getLogger("gvr.gvrng")
 module_logger.debug("Start logging")
@@ -96,14 +106,7 @@ import GvrModel
 import GvrController
 
 def main(handle=None,parent=None):
-    module_logger.debug("main called with: %s,%s" % (handle,parent))
-    # commandline arguments are used in developing only
-    if len(sys.argv) == 3:
-        wfile = sys.argv[1]
-        pfile = sys.argv[2]
-    else:
-        wfile = ''
-        pfile = ''
+    module_logger.debug("main called with: %s,%s" % (handle,parent))        
     try:
         # The abstraction layer on top of the gvr stuff
         model = GvrModel.GvrModel()
@@ -118,7 +121,7 @@ def main(handle=None,parent=None):
         model.set_controller(contr)
         view.set_controller(contr)
         # Now start the GUI
-        contr.start_view(wfile,pfile)# args are optional
+        contr.start_view()# args are optional
     except Exception:
         module_logger.exception("Toplevel error")
         return 1
